#include <stdio.h>
#include "Ethernet.h"


// change MAC and IP address if necessary
static uint8_t mymac[6] = {0x54,0x55,0x58,0x10,0x00,0x24};
static uint8_t myip[4] = {192,168,2,15};
static char baseurl[]="http://192.168.2.15/";
static uint16_t mywwwport =80;

#define BUFFER_SIZE 500
static uint8_t buf[BUFFER_SIZE+1];
#define STR_BUFFER_SIZE 50
static char strbuf[STR_BUFFER_SIZE+1];

Ethernet es = Ethernet();

int nPingCount = 0, nAccessCount=0;

int ledpin = 9;                // LED connected to digital pin 13




// prepare the webpage by writing the data to the tcp send buffer
uint16_t print_webpage(uint8_t *buf, char *param);

void setup(){
    Serial.begin(9600); 
    Serial.println("Setting up webserver");
   
    /*initialize enc28j60*/
    es.E_enc28j60Init(mymac);
    es.E_enc28j60clkout(2); // change clkout from 6.25MHz to 12.5MHz
    delay(10);
       
    /* Magjack leds configuration, see enc28j60 datasheet, page 11 */
    // LEDA=greed LEDB=yellow

    // 0x880 is PHLCON LEDB=on, LEDA=on
    // enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
    es.E_enc28j60PhyWrite(PHLCON,0x880);
    delay(500);
   
    // 0x990 is PHLCON LEDB=off, LEDA=off
    // enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
    es.E_enc28j60PhyWrite(PHLCON,0x990);
    delay(500);

    // 0x880 is PHLCON LEDB=on, LEDA=on
    // enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
    es.E_enc28j60PhyWrite(PHLCON,0x880);
    delay(500);

    // 0x990 is PHLCON LEDB=off, LEDA=off
    // enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
    es.E_enc28j60PhyWrite(PHLCON,0x990);
    delay(500);

    // 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
    // enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);
    es.E_enc28j60PhyWrite(PHLCON,0x476);
    delay(100);
       
    //init the ethernet/ip layer:
    es.E_init_ip_arp_udp_tcp(mymac,myip,80);
}
int light = 0;
void loop()
{
    uint16_t plen, dat_p;
    int8_t cmd;
   
   analogWrite(ledpin, light);
   
    plen = es.E_enc28j60PacketReceive(BUFFER_SIZE, buf);
    // plen will ne unequal to zero if there is a valid packet
    if(plen!=0)
    {
       // arp is broadcast if unknown but a host may also verify
       // the mac address by sending it to a unicast address.
       if(es.E_eth_type_is_arp_and_my_ip(buf,plen))
       {
           es.E_make_arp_answer_from_request(buf);
           return;
       }
   
       // check if ip packets are for us:
       if(es.E_eth_type_is_ip_and_my_ip(buf,plen)==0)
       {
           return;
       }
   
       //reply to PING
       if(buf[IP_PROTO_P]==IP_PROTO_ICMP_V
          && buf[ICMP_TYPE_P]==ICMP_TYPE_ECHOREQUEST_V)
       {
           nPingCount++;
           es.E_make_echo_reply_from_request(buf,plen);
          

           return;
       }
   
       // tcp port www start, compare only the lower byte
       if (buf[IP_PROTO_P]==IP_PROTO_TCP_V
           && buf[TCP_DST_PORT_H_P]== 0
           && buf[TCP_DST_PORT_L_P]== mywwwport)
       {
           if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V)
           {
              // make_tcp_synack_from_syn does already send the syn,ack
              es.E_make_tcp_synack_from_syn(buf);
              return;     
           }
           if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V)
           {
              es.E_init_len_info(buf); // init some data structures
              dat_p=es.E_get_tcp_data_pointer();
              if (dat_p==0) // we can possibly have no data, just ack:
              {
                  if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V)
                  {
                     es.E_make_tcp_ack_from_any(buf);
                  }
                  return;
              }
              
             
             // if (strncmp("/ ",(char *)&(buf[dat_p+4]),2)==0)
              {
                  nAccessCount++;
                  char *param = strstr((char *)&(buf[dat_p]), "param=");
                  if (param)
                  {
                     param+=6; //skip param=
                     uint8_t i=0;
                     //extract param value
                     while(*param &&  *param!=' ' && *param!='&' && i<STR_BUFFER_SIZE)
                     {
                         strbuf[i]=*param;
                         i++;
                         param++;
                     }
                     strbuf[i] = 0;
                     //send page
                     plen=print_webpage(buf, strbuf);
                  }
                  else
                       plen=print_webpage(buf, 0);
                  goto SENDTCP;
              }
             
              SENDTCP:  es.E_make_tcp_ack_from_any(buf);
              // send data
              es.E_make_tcp_ack_with_data(buf,plen);
           }
       }
   }
}


uint16_t print_webpage(uint8_t *buf, char *param)
{
    uint16_t plen;
    // remote commands
    if (strcmp(param, "lightson") == 0) light = 255; 
    else
    if (strcmp(param, "lightdecrease") == 0) light = 50;
    else
    if (strcmp(param, "lightsoff") == 0) light = 0;
    
    // remote sensor readings
    if (strcmp(param, "getlightsensor") == 0)
    {
      int potPin = 0;                                  // use microcontroller pin 0
      int val = analogRead(potPin);                    // read analog pin
      char szS1[70] = {0}; sprintf(szS1, "%d", val);   // prepare answer
      plen=es.E_fill_tcp_data(buf,plen,szS1 );         // 
      return(plen);                                    //
    }
    
    // show page content
    char szS1[70] = {0};sprintf(szS1, "<li>Server pings:%d Access Count:%d</li>", nPingCount,nAccessCount);
    char szS2[50] = {0};sprintf(szS2, "<li>Param:[%s]", param?param:"");
    char szS3[50] = {0};sprintf(szS3, "<li>Light sensor: %d</li>", val);

    plen=es.E_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
    plen=es.E_fill_tcp_data_p(buf,plen,PSTR("<font face=\"tahoma\"><b>Webserver running on Atmega168-P20</b>"));
    //active data
    plen=es.E_fill_tcp_data(buf,plen,szS1 );
    plen=es.E_fill_tcp_data(buf,plen,szS2 );
    plen=es.E_fill_tcp_data_p(buf,plen,PSTR("<hr>Environment sensor data:<br>"));
    //active sensor content
    plen=es.E_fill_tcp_data(buf,plen,szS3 );
    
    //
    plen=es.E_fill_tcp_data_p(buf,plen,PSTR("</font></body>"));
   

    return(plen);
}
